var Lc;
(function (Lc) {
    (function (LogLevel) {
        LogLevel[LogLevel["Trace"] = 0] = "Trace";
        LogLevel[LogLevel["Debug"] = 1] = "Debug";
        LogLevel[LogLevel["Info"] = 2] = "Info";
        LogLevel[LogLevel["Warn"] = 3] = "Warn";
        LogLevel[LogLevel["Error"] = 4] = "Error";
        LogLevel[LogLevel["Fatal"] = 5] = "Fatal";
        LogLevel[LogLevel["Off"] = 99] = "Off";
    })(Lc.LogLevel || (Lc.LogLevel = {}));
    var LogLevel = Lc.LogLevel;
    var LogManager;
    (function (LogManager) {
        var consoleOutputLevel = LogLevel.Info;
        var consoleOutputMessageFormatter;
        var logListener = [];
        function writeLogErrorToConsole(error) {
            try {
                if (!window.console)
                    return;
                if (window.console.error)
                    window.console.error(error.message, error);
                else if (window.console.log)
                    window.console.log(error.message, error);
            }
            catch (e) {
            }
        }
        function getConsoleOutputLevel() { return consoleOutputLevel; }
        LogManager.getConsoleOutputLevel = getConsoleOutputLevel;
        function setConsoleOutputLevel(value) { consoleOutputLevel = value; }
        LogManager.setConsoleOutputLevel = setConsoleOutputLevel;
        function getConsoleOutputMessageFormatter() { return consoleOutputMessageFormatter; }
        LogManager.getConsoleOutputMessageFormatter = getConsoleOutputMessageFormatter;
        function setConsoleOutputMessageFormatter(value) { consoleOutputMessageFormatter = value; }
        LogManager.setConsoleOutputMessageFormatter = setConsoleOutputMessageFormatter;
        function addLogListener(listener) {
            if (typeof listener !== "function")
                return;
            var idx = logListener.indexOf(listener);
            if (idx < 0)
                logListener.push(listener);
        }
        LogManager.addLogListener = addLogListener;
        function removeLogListener(listener) {
            if (typeof listener !== "function")
                return;
            var idx = logListener.indexOf(listener);
            if (idx >= -1)
                logListener.splice(idx, 1);
        }
        LogManager.removeLogListener = removeLogListener;
        function createLogger(name) {
            return new Internal.Logger(name);
        }
        LogManager.createLogger = createLogger;
        function log(logData) {
            try {
                if (logData == null || logData.logger == null || logData.level == null)
                    return;
                if (!window.console || logData.level < consoleOutputLevel || consoleOutputLevel === LogLevel.Off)
                    return;
                var message = logData.message || "";
                logData.args = logData.args || [];
                if (typeof message !== "string") {
                    logData.args.push(message);
                    if (message instanceof Error)
                        message = message.message != null && message.message !== "" ? message.message : "Unknown Error";
                    else
                        message = message.toString();
                }
                logData.message = message;
                if (logData.args.length < 1)
                    logData.args = null;
                var consoleFunction = void 0;
                switch (logData.level) {
                    case LogLevel.Trace:
                        consoleFunction = window.console.log;
                        break;
                    case LogLevel.Debug:
                        consoleFunction = window.console.debug;
                        break;
                    case LogLevel.Info:
                        consoleFunction = window.console.info;
                        break;
                    case LogLevel.Warn:
                        consoleFunction = window.console.warn;
                        break;
                    case LogLevel.Error:
                        consoleFunction = window.console.error;
                        break;
                    case LogLevel.Fatal:
                        consoleFunction = window.console.error;
                        break;
                    default: consoleFunction = window.console.log;
                }
                if (consoleFunction == null)
                    consoleFunction = window.console.log;
                if (consoleFunction != null) {
                    var d = new Date();
                    var messageArg = d.toLocaleDateString() + " " + d.toLocaleTimeString() + "|" + logData.logger.name() + ": " + logData.message;
                    if (typeof consoleOutputMessageFormatter === "function")
                        messageArg = consoleOutputMessageFormatter(logData);
                    var newArgs = [messageArg];
                    if (logData.args)
                        newArgs.push.apply(newArgs, logData.args);
                    Function.prototype.apply.call(consoleFunction, window.console, newArgs);
                }
            }
            catch (e) {
                writeLogErrorToConsole(e);
            }
            logListener.forEach(function (listener) {
                try {
                    listener(logData);
                }
                catch (e2) {
                    writeLogErrorToConsole(e2);
                }
            });
        }
        LogManager.log = log;
    })(LogManager = Lc.LogManager || (Lc.LogManager = {}));
    var Internal;
    (function (Internal) {
        var Logger = (function () {
            function Logger(name) {
                this._name = name;
            }
            Logger.prototype.name = function () { return this._name; };
            Logger.prototype.log = function (level, message, args) {
                LogManager.log({
                    logger: this,
                    level: level,
                    message: message,
                    args: args
                });
            };
            Logger.prototype.trace = function (message) {
                var args = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    args[_i - 1] = arguments[_i];
                }
                this.log(LogLevel.Trace, message, args);
            };
            Logger.prototype.debug = function (message) {
                var args = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    args[_i - 1] = arguments[_i];
                }
                this.log(LogLevel.Debug, message, args);
            };
            Logger.prototype.info = function (message) {
                var args = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    args[_i - 1] = arguments[_i];
                }
                this.log(LogLevel.Info, message, args);
            };
            Logger.prototype.warn = function (message) {
                var args = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    args[_i - 1] = arguments[_i];
                }
                this.log(LogLevel.Warn, message, args);
            };
            Logger.prototype.error = function (message) {
                var args = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    args[_i - 1] = arguments[_i];
                }
                this.log(LogLevel.Error, message, args);
            };
            Logger.prototype.fatal = function (message) {
                var args = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    args[_i - 1] = arguments[_i];
                }
                this.log(LogLevel.Fatal, message, args);
            };
            return Logger;
        }());
        Internal.Logger = Logger;
        Internal.logger = LogManager.createLogger("LcFramework");
    })(Internal = Lc.Internal || (Lc.Internal = {}));
    var Bundling = (function () {
        function Bundling() {
        }
        Bundling.isBundleLoaded = function (bundleName) {
            for (var i = 0; i < Bundling.loadedBundles.length; i++) {
                if (Bundling.loadedBundles[i] === bundleName)
                    return true;
            }
            return false;
        };
        Bundling.getBundleMetaData = function (bundleName) {
            if (Bundling.bundleMetaDataFile == null)
                throw new Error("Bundling.bundleMetaDataFile == null");
            for (var i = 0; i < Bundling.bundleMetaDataFile.bundles.length; i++) {
                var item = Bundling.bundleMetaDataFile.bundles[i];
                if (item.name === bundleName)
                    return item;
            }
            throw new Error("72893: Bundle not found: " + bundleName);
        };
        Bundling.getLoadedBundleNames = function () {
            return Bundling.loadedBundles.concat();
        };
        Bundling.getNotLoadedBundleNames = function () {
            var result = new Array();
            if (Bundling.bundleMetaDataFile == null || Bundling.bundleMetaDataFile.bundles == null)
                return result;
            var bundles = Bundling.bundleMetaDataFile.bundles;
            for (var i = 0; i < bundles.length; i++) {
                var bundleName = bundles[i].name;
                if (!Bundling.isBundleLoaded(bundleName))
                    result.push(bundleName);
            }
            return result;
        };
        Bundling.requireBundle = function (bundleName) {
            var result = new Promise(function (resolve, reject) {
                if (Bundling.isBundleLoaded(bundleName)) {
                    setTimeout(function () { return resolve(); }, 0);
                    return;
                }
                var innerreject = function (err) {
                    Bundling.requireBundleRequests.splice(0, 1);
                    reject(err);
                };
                var innerRequireBundleCallback = function () {
                    Bundling.requireBundleRequests[0].resolve();
                    Bundling.requireBundleRequests.splice(0, 1);
                    if (Bundling.requireBundleRequests.length > 0)
                        Bundling.innerRequireBundle(Bundling.requireBundleRequests[0].bundleName).then(function () { innerRequireBundleCallback(); }, function (e) { innerreject(e); });
                };
                Bundling.requireBundleRequests.push({ bundleName: bundleName, resolve: resolve, reject: reject });
                if (Bundling.requireBundleRequests.length === 1)
                    Bundling.innerRequireBundle(Bundling.requireBundleRequests[0].bundleName).then(function () { innerRequireBundleCallback(); }, function (e) { innerreject(e); });
            }).catch(function (e) { Lc.Internal.logger.error("requireBundle for '" + (bundleName || "") + "' failed.", e); throw e; });
            return result;
        };
        Bundling.innerRequireBundle = function (bundleName) {
            var result = new Promise(function (resolve, reject) {
                if (Bundling.isBundleLoaded(bundleName)) {
                    setTimeout(function () { return resolve(); }, 0);
                    return;
                }
                if (Bundling.bundleMetaDataFile == null) {
                    $.get("./Bundles/BundleMetaData.txt", function (data) {
                        Bundling.bundleMetaDataFile = JSON.parse(data);
                        Bundling.requireBundleAfterMetaDataLoaded(bundleName).then(function () { resolve(); }, function (e) { reject(e); });
                        Bundling.isDebug = Bundling.bundleMetaDataFile.isDebug;
                    }).fail(function (jqXhr, error, message) {
                        var msg = "Fehler beim Laden der Bundle-Metadaten: " + message;
                        Lc.Internal.logger.error(msg);
                        window.alert(msg);
                    });
                }
                else {
                    Bundling.requireBundleAfterMetaDataLoaded(bundleName).then(function () { resolve(); }, function (e) { reject(e); });
                }
            });
            return result;
        };
        Bundling.requireBundleAfterMetaDataLoaded = function (bundleName) {
            var result = new Promise(function (resolve, reject) {
                var bundleMetaData = Bundling.getBundleMetaData(bundleName);
                var bundleNames = new Array();
                bundleNames = bundleNames.concat(bundleMetaData.dependencies.filter(function (name) { return !Bundling.isBundleLoaded(name); }));
                bundleNames.push(bundleMetaData.name);
                if (Bundling.bundleMetaDataFile.isDebug) {
                    var jsFileNames = new Array();
                    var cssFileNames = new Array();
                    var htmFileNames = new Array();
                    bundleNames.map(function (depBundleName) { return Bundling.getBundleMetaData(depBundleName); }).forEach(function (depBundle) {
                        if (depBundle.debugFileListJs != null)
                            jsFileNames = jsFileNames.concat(depBundle.debugFileListJs);
                        if (depBundle.debugFileListCss != null)
                            cssFileNames = cssFileNames.concat(depBundle.debugFileListCss);
                        if (depBundle.debugFileListHtm != null)
                            htmFileNames = htmFileNames.concat(depBundle.debugFileListHtm);
                    });
                    Bundling.addJsOrCssFiles(jsFileNames, false)
                        .then(function () { return Bundling.addJsOrCssFiles(cssFileNames, true); })
                        .then(function () { return Bundling.addTemplateFiles(htmFileNames); })
                        .then(function () { Bundling.loadedBundles = Bundling.loadedBundles.concat(bundleNames); })
                        .then(function () { resolve(); }, function (e) { reject(e); });
                }
                else {
                    var bundleFileNames = bundleNames.map(function (name) { return "./Bundles/" + name + ".txt"; });
                    var numberOfBundles = bundleNames.length;
                    var numberOfLoadedBundles = 0;
                    var loadedBundleFiles = new Array(numberOfBundles);
                    bundleFileNames.forEach(function (fileName, idx) {
                        $.getJSON(fileName, function (data) {
                            loadedBundleFiles[idx] = data;
                            numberOfLoadedBundles++;
                            if (numberOfLoadedBundles >= numberOfBundles) {
                                var js = "";
                                var css = "";
                                var htm = "";
                                for (var i = 0; i < loadedBundleFiles.length; i++) {
                                    var item = loadedBundleFiles[i];
                                    js += (item.js || "") + "\r\n";
                                    css += (item.css || "") + "\r\n";
                                    htm += (item.htm || "") + "\r\n";
                                }
                                if (js !== "") {
                                    Bundling.loadedJsBytes = (Bundling.loadedJsBytes || 0) + js.length;
                                    var jsElement = document.createElement("script");
                                    jsElement.setAttribute("type", "text/javascript");
                                    jsElement.text = js;
                                    document.getElementsByTagName("head")[0].appendChild(jsElement);
                                }
                                if (css !== "") {
                                    Bundling.loadedCssBytes = (Bundling.loadedCssBytes || 0) + css.length;
                                    var cssElement = document.createElement("style");
                                    cssElement.setAttribute("type", "text/css");
                                    cssElement.innerHTML = css;
                                    document.getElementsByTagName("head")[0].appendChild(cssElement);
                                }
                                if (htm !== "") {
                                    Bundling.loadedTemplatesBytes = (Bundling.loadedTemplatesBytes || 0) + htm.length;
                                    Lc.TemplateLoader.addTemplate(htm);
                                }
                                Bundling.loadedBundles = Bundling.loadedBundles.concat(bundleNames);
                                resolve();
                            }
                        }).fail(function () {
                            reject(Bundling.createCriticalBundleError("Load bundle file failed: " + (fileName || "")));
                        });
                    });
                }
            });
            return result;
        };
        Bundling.addTemplateFiles = function (fileNames) {
            var result = new Promise(function (resolve, reject) {
                if (fileNames == null || fileNames.length === 0) {
                    resolve();
                    return;
                }
                Promise.all(fileNames.map(function (fileName) { return Lc.Utils.httpGet(fileName, undefined, false); })).then(function (values) {
                    try {
                        values.forEach(function (content) {
                            Lc.TemplateLoader.addTemplate(content);
                        });
                    }
                    catch (e) {
                        reject(Bundling.createCriticalBundleError("Load template file failed: " + Lc.Utils.getErrorMessage(e)));
                    }
                    resolve();
                }, function (e) {
                    reject(Bundling.createCriticalBundleError("Load template file failed: " + Lc.Utils.getErrorMessage(e)));
                });
            });
            return result;
        };
        Bundling.addJsOrCssFiles = function (fileNames, isCss) {
            var result = new Promise(function (resolve, reject) {
                if (fileNames == null || fileNames.length === 0) {
                    resolve();
                    return;
                }
                var numberOfFiles = fileNames.length;
                var fileIdx = 0;
                var doAfterLoad = function () {
                    fileIdx++;
                    if (fileIdx < numberOfFiles) {
                        var fileName = fileNames[fileIdx];
                        Bundling.addJsOrCssFileToDocumentHead(fileName, isCss).then(function () { return doAfterLoad(); }, function (e) { return reject(e); });
                    }
                    else
                        resolve();
                };
                Bundling.addJsOrCssFileToDocumentHead(fileNames[0], isCss).then(function () { return doAfterLoad(); }, function (e) { return reject(e); });
            });
            return result;
        };
        Bundling.addJsOrCssFileToDocumentHead = function (fileName, isCss) {
            var result = new Promise(function (resolve, reject) {
                var el = document.createElement(isCss ? "link" : "script");
                if (isCss) {
                    el.setAttribute("rel", "stylesheet");
                    el.setAttribute("href", fileName);
                }
                else {
                    el.setAttribute("type", "text/javascript");
                    el.setAttribute("src", fileName);
                }
                el.onload = function () {
                    resolve();
                };
                el.onerror = function () {
                    reject(Bundling.createCriticalBundleError("Load of file '" + (fileName || "") + "' failed."));
                };
                document.getElementsByTagName("head")[0].appendChild(el);
            });
            return result;
        };
        Bundling.createCriticalBundleError = function (message) {
            var error = new Error(message);
            error.name = "CriticalBundleError";
            Lc.Internal.logger.error(error.message, error);
            return error;
        };
        Bundling.loadedBundles = new Array();
        Bundling.requireBundleRequests = new Array();
        Bundling.bundleMetaDataFile = null;
        Bundling.isDebug = null;
        Bundling.loadedTemplatesBytes = null;
        Bundling.loadedJsBytes = null;
        Bundling.loadedCssBytes = null;
        return Bundling;
    }());
    Lc.Bundling = Bundling;
    function ready(initialSettings) {
        var result = new Promise(function (resolve) {
            Lc.Internal.logger.info("LcFramework waiting for ready event call...");
            initialSettings = initialSettings || {};
            var runningInCordovaEnvirement = (function () {
                if (typeof cordova === "undefined" || cordova === null)
                    return false;
                var res = ((document.URL.indexOf("http://") === -1 && document.URL.indexOf("https://") === -1)
                    || ((window.location.search || "").indexOf("cordovadebug=1") > -1));
                return res;
            })();
            if (runningInCordovaEnvirement) {
                document.addEventListener("deviceready", function () {
                    document.addEventListener("backbutton", function () {
                        if (Lc.Environment.isAndroid()
                            && typeof navigator.app !== "undefined"
                            && typeof navigator.app.backHistory === "function") {
                            navigator.app.backHistory();
                        }
                        ;
                    }, false);
                    resolve();
                }, false);
            }
            else {
                $().ready(function () { resolve(); });
            }
        }).then(function () {
            Lc.Internal.logger.info("LcFramework loading bundle LcCore...");
            return Bundling.requireBundle("LcCore");
        }).then(function () {
            return new Promise(function (resolve) {
                Lc.Internal.logger.info("LcFramework init framework...");
                Lc.Framework = new Internal.Framework(initialSettings);
                Lc.Environment = new Internal.Environment();
                Lc.Convert = new Internal.Convert();
                Lc.Culture = new Internal.Culture();
                Lc.UiLock = new Internal.UiLock();
                Lc.FileUtils = new Internal.FileUtils();
                Lc.CameraDevice = new Internal.Camera();
                Lc.BarcodeScanner = new Internal.BarcodeScanner();
                Lc.Clipboard = new Internal.Clipboard();
                Lc.WindowManager = new Internal.WindowManager();
                Lc.TouchMoveManager = new Internal.TouchMoveManager();
                var lcDebugUrl = Lc.Framework.debugUrl;
                if (lcDebugUrl && ((window.location.search || "").indexOf("isLcDebugUrl=1") === -1)) {
                    var lcDebugUrlDialog = $("<div class=\"lcDebugUrlDialog\">" +
                        "<h1>Redirect to DebugURL:</h1>" +
                        "<div><b>" + Lc.StringExt.htmlEncode(lcDebugUrl) + "</b><br/><br/></div>" +
                        "<div class=\"debugUrlButton\" id=\"debugUrlRedirectButton\">Redirect</div><br/>" +
                        "<div class=\"debugUrlButton\" id=\"debugUrlCancelButton\">Cancel</div><br/>" +
                        "<div class=\"debugUrlButton\" id=\"debugUrlCancelAndRemoveUrlButton\">Cancel<br/>(and remove DebugUrl)</div><br/>" +
                        "</div>");
                    $("body").append(lcDebugUrlDialog);
                    Lc.setTouchClick(lcDebugUrlDialog.find("#debugUrlRedirectButton")[0], function () {
                        lcDebugUrl = lcDebugUrl + (Lc.StringExt.contains(lcDebugUrl, "?") ? "&" : "?") + "isLcDebugUrl=1";
                        window.location.href = lcDebugUrl;
                    });
                    Lc.setTouchClick(lcDebugUrlDialog.find("#debugUrlCancelButton")[0], function () {
                        lcDebugUrlDialog.remove();
                        resolve();
                    });
                    Lc.setTouchClick(lcDebugUrlDialog.find("#debugUrlCancelAndRemoveUrlButton")[0], function () {
                        lcDebugUrlDialog.remove();
                        Lc.Framework.debugUrl = "";
                        resolve();
                    });
                }
                else
                    resolve();
            });
        }).then(function () {
            if (!Lc.StringExt.isNullOrWhiteSpace(initialSettings.initialBundleName)) {
                Lc.Internal.logger.info("LcFramework loading initial bundle '" + initialSettings.initialBundleName + "'...");
                return Bundling.requireBundle(initialSettings.initialBundleName);
            }
            ;
            return Promise.resolve(null);
        }).catch(function (e) {
            Lc.Internal.logger.error(e);
            throw e;
        });
        return result;
    }
    Lc.ready = ready;
})(Lc || (Lc = {}));
